"""
The basic part of this function is copied from https://pypi.org/project/s3etag/.
s3etag was a command line tool, so I just cut it out as a function.
Original s3etag license is Apache License 2.0 .
"""
import hashlib

BUF_SIZE = 1 * 1024 * 1024


def compare_s3etag(target_filename: str, etag: str, chunksize: int):
    if '-' in etag:
        md5sum = _calc_chunked_s3etag(target_filename, chunksize)
    else:
        md5sum = _calc_s3etag(target_filename)
    return etag == md5sum


def _calc_chunked_s3etag(filename: str, chunksize: int) -> str:
    """Compute and print Etag for a file"""
    count = 0
    dgst_part = hashlib.md5()
    dgst_whole = hashlib.md5()
    try:
        with open(filename, "rb") as fh:
            while True:
                buf = fh.read(chunksize)
                if len(buf) < 1:
                    break
                count = count + 1
                dgst_part = hashlib.md5(buf)
                dgst_whole.update(dgst_part.digest())
    except OSError:
        raise

    etag = "{}-{}".format(dgst_whole.hexdigest(), count) if count > 1 else dgst_part.hexdigest()
    return etag


def _calc_s3etag(filename: str):
    dgst = hashlib.md5()
    try:
        with open(filename, 'rb') as fh:
            while True:
                buf = fh.read(BUF_SIZE)
                if len(buf) < 1:
                    break
                dgst.update(buf)
    except OSError:
        raise

    return dgst.hexdigest()
